@@ -5,21 +5,23 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '2.67';
+our $VERSION = '2.69';
 
 use Carp ();
 use Log::Dispatch::Types;
-use Log::Dispatch::Vars qw( %CanonicalLevelNames @OrderedLevels );
+use Log::Dispatch::Vars qw( %CanonicalLevelNames %LevelNamesToNumbers );
 use Module::Runtime qw( use_package_optimistically );
 use Params::ValidationCompiler qw( validation_for );
 
 use base qw( Log::Dispatch::Base );
 
 BEGIN {
-    foreach my $l ( keys %CanonicalLevelNames ) {
-        my $sub = sub {
+    for my $l ( keys %CanonicalLevelNames ) {
+        my $level_num = $LevelNamesToNumbers{$l};
+        my $sub       = sub {
             my $self = shift;
-            $self->log(
+            $self->_log_with_num(
+                $level_num,
                 level   => $CanonicalLevelNames{$l},
                 message => @_ > 1 ? "@_" : $_[0],
             );
@@ -73,7 +75,7 @@ BEGIN {
                 #   [ 'File',   min_level => 'debug', filename => 'logfile' ],
                 #   [ 'Screen', min_level => 'warning' ]
                 # ]
-                foreach my $arr ( @{$outputs} ) {
+                for my $arr ( @{$outputs} ) {
                     die "expected arrayref, not '$arr'"
                         unless ref $arr eq 'ARRAY';
                     $self->_add_output( @{$arr} );
@@ -149,12 +151,27 @@ sub log {
     my $self = shift;
     my %p    = @_;
 
-    return unless $self->would_log( $p{level} );
+    my $level_num = $self->_level_as_number( $p{level} );
+    return unless defined $level_num;
+
+    return $self->_log_with_num( $level_num, %p );
 
-    $self->_log_to_outputs( $self->_prepare_message(%p) );
 }
 ## use critic
 
+sub _log_with_num {
+    my $self      = shift;
+    my $level_num = shift;
+    my %p         = @_;
+
+    return unless $self->_would_log($level_num);
+
+    $p{message} = $self->_prepare_message(%p);
+    $_->_log_with_num( $level_num, %p ) for values %{ $self->{outputs} };
+
+    return;
+}
+
 sub _prepare_message {
     my $self = shift;
     my %p    = @_;
@@ -165,23 +182,23 @@ sub _prepare_message {
     $p{message} = $self->_apply_callbacks(%p)
         if $self->{callbacks};
 
-    return %p;
+    return $p{message};
 }
 
 sub _log_to_outputs {
     my $self = shift;
     my %p    = @_;
 
-    foreach ( keys %{ $self->{outputs} } ) {
-        $p{name} = $_;
-        $self->_log_to(%p);
+    for ( values %{ $self->{outputs} } ) {
+        $_->log(%p);
     }
 }
 
 sub log_and_die {
     my $self = shift;
+    my %p    = @_;
 
-    my %p = $self->_prepare_message(@_);
+    $p{message} = $self->_prepare_message(%p);
 
     $self->_log_to_outputs(%p) if $self->would_log( $p{level} );
 
@@ -194,8 +211,6 @@ sub log_and_croak {
     $self->log_and_die(@_);
 }
 
-my @CARP_NOT = __PACKAGE__;
-
 sub _die_with_message {
     my $self = shift;
     my %p    = @_;
@@ -241,29 +256,22 @@ sub output {
     return $self->{outputs}{$name};
 }
 
-sub level_is_valid {
-    shift;
-    my $level = shift;
-
-    if ( !defined $level ) {
-        Carp::croak('Logging level was not provided');
-    }
-
-    if ( $level =~ /\A[0-9]+\z/ && $level <= $#OrderedLevels ) {
-        return $OrderedLevels[$level];
-    }
-
-    return $CanonicalLevelNames{$level};
-}
-
 sub would_log {
     my $self  = shift;
     my $level = shift;
 
-    return 0 unless $self->level_is_valid($level);
+    my $level_num = $self->_level_as_number($level);
+    return 0 unless defined $level_num;
 
-    foreach ( values %{ $self->{outputs} } ) {
-        return 1 if $_->_should_log($level);
+    return $self->_would_log($level_num);
+}
+
+sub _would_log {
+    my $self      = shift;
+    my $level_num = shift;
+
+    for ( values %{ $self->{outputs} } ) {
+        return 1 if $_->_should_log($level_num);
     }
 
     return 0;
@@ -298,7 +306,7 @@ Log::Dispatch - Dispatches messages to one or more outputs
 
 =head1 VERSION
 
-version 2.67
+version 2.69
 
 =head1 SYNOPSIS
 
@@ -448,16 +456,15 @@ C<log()> method.
 Has the same behavior as calling C<log()> but calls
 C<_die_with_message()> at the end.
 
-=head2 $dispatch->log_and_croak( level => $, message => $ or \& )
-
-This method adjusts the C<$Carp::CarpLevel> scalar so that the croak
-comes from the context in which it is called.
-
 You can throw exception objects by subclassing this method.
 
 If the C<carp_level> parameter is present its value will be added to
 the current value of C<$Carp::CarpLevel>.
 
+=head2 $dispatch->log_and_croak( level => $, message => $ or \& )
+
+A synonym for C<$dispatch->log_and_die()>.
+
 =head2 $dispatch->log_to( name => $, level => $, message => $ )
 
 Sends the message only to the named object. Note: this will not properly
@@ -711,7 +718,7 @@ Dave Rolsky <autarch@urth.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Anirvan Chatterjee Carsten Grohmann Doug Bell Graham Ollis Gregory Oschwald Johann Rolschewski Jonathan Swartz Karen Etheridge Kerin Millar Kivanc Yazan Konrad Bucheli Michael Schout Olaf Alders Olivier Mengué Rohan Carly Ross Attrill Salvador Fandiño Slaven Rezic Steve Bertrand Whitney Jackson
+=for stopwords Anirvan Chatterjee Carsten Grohmann Doug Bell Graham Knop Ollis Gregory Oschwald hartzell Johann Rolschewski Jonathan Swartz Karen Etheridge Kerin Millar Kivanc Yazan Konrad Bucheli Michael Schout Olaf Alders Olivier Mengué Rohan Carly Ross Attrill Salvador Fandiño Sergey Leschenko Slaven Rezic Steve Bertrand Whitney Jackson
 
 =over 4
 
@@ -729,6 +736,10 @@ Doug Bell <doug@preaction.me>
 
 =item *
 
+Graham Knop <haarg@haarg.org>
+
+=item *
+
 Graham Ollis <plicease@cpan.org>
 
 =item *
@@ -737,6 +748,10 @@ Gregory Oschwald <goschwald@maxmind.com>
 
 =item *
 
+hartzell <hartzell@alerce.com>
+
+=item *
+
 Johann Rolschewski <jorol@cpan.org>
 
 =item *
@@ -785,6 +800,10 @@ Salvador Fandiño <sfandino@yahoo.com>
 
 =item *
 
+Sergey Leschenko <sergle.ua@gmail.com>
+
+=item *
+
 Slaven Rezic <srezic@cpan.org>
 
 =item *
@@ -799,7 +818,7 @@ Whitney Jackson <whitney.jackson@baml.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2017 by Dave Rolsky.
+This software is Copyright (c) 2019 by Dave Rolsky.
 
 This is free software, licensed under:
 
