@@ -1,8 +1,11 @@
 package Log::Dispatch::TestUtil;
-use Data::Dumper;
+
 use strict;
 use warnings;
-use base qw(Exporter);
+
+use Data::Dumper;
+
+use Exporter qw( import );
 
 our @EXPORT_OK = qw(
     cmp_deeply
@@ -19,8 +22,8 @@ sub cmp_deeply {
 sub dump_one_line {
     my ($value) = @_;
 
-    return Data::Dumper->new( [$value] )->Indent(0)->Sortkeys(1)->Quotekeys(0)
-        ->Terse(1)->Dump();
+    return Data::Dumper->new( [$value] )->Indent(0)->Sortkeys(1)
+        ->Quotekeys(0)->Terse(1)->Dump();
 }
 
 1;
